#
# create a library in the dyncam namespace
#
# parameters
# IDE_FOLDER : folder name for visual studio (e.g. dyncam/Cams/TestCam)
# SOURCES : sources files for the library
# PUBLIC_LIBRARIES:
# PRIVATE_COMPILE_FLAGS: compile flags for the library. Will not be exported.
# EXPORT_NAME: export name for the dyncam:: target export
# TARGET: target name
#
# create a target associated to <NAME>
# libraries are installed under CMAKE_INSTALL_FULL_LIBDIR by default
#
function(dyncam_library)
    cmake_parse_arguments(DYNCAM_LIB
            "DISABLE_INSTALL" # keep that in case we want to support installation one day
            "TARGET;EXPORT_NAME"
            "SOURCES;PUBLIC_LIBRARIES;PRIVATE_COMPILE_FLAGS;PUBLIC_INCLUDE_DIRS;PRIVATE_INCLUDE_DIRS"
            "IDE_FOLDER"
            ${ARGN}
            )

    set(_NAME ${DYNCAM_LIB_TARGET})

    add_library(${_NAME} STATIC ${DYNCAM_LIB_SOURCES})

    set_property(TARGET ${_NAME} PROPERTY CXX_STANDARD_REQUIRED ON)
    set_property(TARGET ${_NAME} PROPERTY CXX_STANDARD 14)


    target_include_directories(${_NAME} SYSTEM PUBLIC
            ${DYNCAM_COMMON_INCLUDE_DIRS}
            )

    #target_compile_options(${_NAME} PRIVATE ${DYNCAM_COMPILE_CXXFLAGS} ${DYNCAM_LIB_PRIVATE_COMPILE_FLAGS})
    target_link_libraries(${_NAME} PUBLIC ${DYNCAM_LIB_PUBLIC_LIBRARIES})

    # link libcore to everything except itself
    if(NOT ${_NAME} STREQUAL "core")
        target_link_libraries(${_NAME} PUBLIC core)
    endif()

    target_include_directories(${_NAME}
            PUBLIC  ${DYNCAM_LIB_PUBLIC_INCLUDE_DIRS} ${PROJECT_SOURCE_DIR}
            PRIVATE ${DYNCAM_LIB_PRIVATE_INCLUDE_DIRS}
            )
    # Add all Abseil targets to a a folder in the IDE for organization.
    set_property(TARGET ${_NAME} PROPERTY FOLDER ${DYNCAM_LIB_IDE_FOLDER})

    if (DYNCAM_LIB_EXPORT_NAME)
        add_library(dyncam::${DYNCAM_LIB_EXPORT_NAME} ALIAS ${_NAME})
    endif ()
endfunction()

#list(APPEND _SOURCE
#)
#
#dyncam_library(
#        TARGET
#        tgt
#
#        SOURCES
#        ${}
#
#        PRIVATE_INCLUDE_DIRS
#
#        PUBLIC_LIBRARIES
#
#)