
#ifndef DYNCAM_FFMPEGPLAYER_H
#define DYNCAM_FFMPEGPLAYER_H

#include "../../ReactDomain.h"

#include <vector>
#include <string>
#include "src/Core/ImageStream.h"

#include <opencv2/opencv.hpp>

extern "C" {
#include <libavutil/opt.h>
#include <libavcodec/avcodec.h>
#include <libavutil/channel_layout.h>
#include <libavutil/common.h>
#include <libavutil/imgutils.h>
#include <libavutil/mathematics.h>
#include <libavutil/samplefmt.h>
#include <libswscale/swscale.h>
#include <libavformat/avformat.h>
}

#include "../BaseCam.h"

class FFmpegPlayer
{
public:
    FFmpegPlayer( std::string filename, int openCVType );
    ~FFmpegPlayer();

    bool Open();
    double processFrame(cv::Mat& out);
    void Close();

private:
    int decode_packet(cv::Mat& mat);
    int open_codec_context(int *stream_idx);

private:
    std::string m_filename;
    std::ifstream m_filestream;

    int m_openCVType;
    unsigned int m_bpp;

    // FFmpeg codecs and stuff to read the file
    AVFormatContext* m_formatCtx;
    AVCodecContext* m_codecCtx;
    int m_width, m_height;
    AVPixelFormat m_pixelFormat;
    AVStream* m_stream;

    uint8_t* m_videoData[4] = {NULL};
    int m_videoLinesize[4];
    int m_videoBufferSize;

    AVFrame* m_frame;
    AVPacket m_packet;

    // use SWscale to recover the original image from ffmpeg
    SwsContext* swCtx;
};

class FFmpegCamPlayer : public BaseCam
{
public:
    FFmpegCamPlayer( std::string filePrefix, bool playInRealtime = true );
    ~FFmpegCamPlayer();

    virtual bool Open() override;
    virtual void Close() override;

    virtual std::set<StreamTypes> getStreamCapabilities() override;

private:
    std::map<StreamTypes, FFmpegPlayer> m_streamPlayers;
    std::set<StreamTypes> m_streams;
    bool m_playInRealtime;

    std::thread processingThread;
    std::atomic<bool> doProcessing{false};

    std::map<StreamTypes, EventSourceT<cv::Mat>> m_eventSources;
};



#endif // DYNCAM_FFMPEGPLAYER_H
