//
// Created by ascadian on 12/2/17.
//

#include <iostream>
#include "ClientBase.h"
#include "Connection.h"

namespace dyncam {

using boost::asio::ip::tcp;

ClientBase::ClientBase() :
    m_resolver(m_io_service){
}

ClientBase::~ClientBase() {
    std::cout << "Client: Stopping" << std::endl;
    stop();
}

void ClientBase::start(const std::string address, const std::string port) {
    auto endpoint_iterator = m_resolver.resolve({address, port});
    m_endpoint = endpoint_iterator->endpoint();
    start(m_endpoint);
}

void ClientBase::start(const boost::asio::ip::basic_endpoint<tcp> endpoint) {
    m_connection = std::make_unique<Connection>(m_io_service);
    m_connection->socket().async_connect(endpoint,
        [this](boost::system::error_code ec) {
            if (ec) {
                std::cerr << "Client: Could not connect: " << ec.message()
                          << std::endl;
            } else {
                std::cout << "Client: Connected" << std::endl;
                on_connected();
            }
        });

    m_thread = std::thread([&service = m_io_service]() {
        std::cout << "Client: starting asio service" << std::endl;
        boost::asio::io_context::work idleWork(service);
        service.run();
        std::cout << "Client: work done" << std::endl;
    });
}

void ClientBase::stop() {
    try {
        m_io_service.stop();
        m_thread.join();
    } catch (std::exception &e) {
        std::cerr << "Client: Exception: " << e.what() << "\n";
    }
}

} // namespace dyncam {