//
// Created by ascadian on 9/24/18.
//

#include "CompressedStreamSession.h"
#include "src/Cams/NetCam/CompressedStream.h"
#include "src/Core/ImageStream.h"
#include "RequestStreams.h"
#include "Connection.h"

namespace dyncam {

CompressedStreamSession::CompressedStreamSession(boost::asio::ip::basic_endpoint<boost::asio::ip::tcp> endpoint,
                                         std::shared_ptr<CompressedStream> stream) :
                                         m_stream(std::move(stream)){

    m_stream->m_innerStream->events = m_stream->m_compression->deCompress(m_eventSource);
    //m_stream->events = stream->;
    m_endpoint = endpoint;
    start(endpoint);
}

void CompressedStreamSession::sendStreamRequest() {
    std::cout << "Client: send compressed stream request" << std::endl;

    // bind connection info in lambda to make sure it stays in scope during async write
    std::shared_ptr<RequestStreams> reqStreamList = std::make_shared<RequestStreams>(
        m_stream->m_innerStream->GetId(),
        StreamTypes::COMPRESSED,
        m_stream->m_compression);

    std::cout << "Client: request: " << *reqStreamList << std::endl;
    m_connection->async_write(*reqStreamList,
                              [this, reqStreamList=reqStreamList] (const boost::system::error_code &e) {
                                  if (e) {
                                      std::cerr << "Client: " << e.message() << std::endl;
                                  } else {
                                      std::cout << "Client: CompressedStreamSession, RequestStreams send" << std::endl;
                                      readStream();
                                  }
                              });
}

void CompressedStreamSession::readStream() {
    std::shared_ptr<cv::Mat> m = std::make_shared<cv::Mat>();
    m_connection->async_read(*m,
        [this, m](const boost::system::error_code &e) mutable {
         if (e) {
             std::cerr << "ClientSess: " << e.message() << std::endl;
         } else {
             //std::cout << "ClientSess received compressed: " << std::endl << m << std::endl;
             m_eventSource << *m;
             readStream();
         }
        });
}

void CompressedStreamSession::on_connected() {
    ClientBase::on_connected();
    sendStreamRequest();
}

} // namespace dyncam
