//
// Created by ascadian on 12/1/17.
//

#pragma once
#include <boost/asio.hpp>

namespace dyncam {

class Connection;
class IntrinsicsStream;
class ImageStream;
class CompressedStream;
class RequestStreams;

// the server has sessions so we have a circular dependency
class StreamServer;

class ServerSession :
    public std::enable_shared_from_this<ServerSession> {
public:
    ServerSession(boost::asio::io_service &io_service, StreamServer *server);

    virtual ~ServerSession();

    void start();

    /// Get the underlying socket. Used for making a connection or for accepting
    /// an incoming connection.
    boost::asio::ip::tcp::socket& socket();

private:

    std::unique_ptr<Connection> m_connection;
    StreamServer *m_server;
    std::vector<ObserverT> m_activeStreams;

    void readRequest();
    void sendStreamColl();
    void startStreaming(std::shared_ptr<RequestStreams> request);
    void imageStream(std::shared_ptr<RequestStreams> request);
    void intrinsicsStream(std::shared_ptr<RequestStreams> request);
    void compressedStream(std::shared_ptr<RequestStreams> request);
};

} // namespace dyncam {