//
// Created by ascadian on 11/30/17.
//

#include "ColorCompression.h"

//quality is in range of 1 to 100
//compr = cv::IMWRITE_JPEG_QUALITY;
//
//comprType = ".png";
//quality = qMin( 9, quality/10 ); // PNG quality is in [0,9] range
//compr = cv::IMWRITE_PNG_COMPRESSION;

namespace dyncam {

ColorCompression::ColorCompression(COMPRESSIONS comp){
    m_compression = comp;
}

EventsT<cv::Mat> ColorCompression::compress(const EventsT<cv::Mat> &inStream) {
    auto trans = Transform(inStream, [compressionType = m_compression] (const cv::Mat &image) {
        int quality;
        int comprParamID;
        std::string compression;



		/*cv::FileStorage fs("test.yml", cv::FileStorage::WRITE);
		fs << "mat" << image;
		fs.release();

		cv::Mat te =cv::Mat();
		cv::FileStorage fs2("test.yml", cv::FileStorage::READ);
		fs2["mat"] >> te;
		fs2.release();

		bool s = true;
		for (int h=0; h<image.rows;++h)
		{
			for (int k=0; k<image.cols;++k)
			{
				if (image.at<cv::Vec4b>(h,k)!=te.at<cv::Vec4b>(h,k))
				{
					s = false;
				}
			}
		}
		std::cout << "same file: " << s << std::endl;*/

        switch (compressionType) {
            case COMPRESSIONS::PNG:
                quality = 0; // PNG quality is in [0,9] range, A higher value means a smaller size and longer compression time. Default value is 3.
                comprParamID = cv::IMWRITE_PNG_COMPRESSION;
                compression = ".png";
                break;
            case COMPRESSIONS::JPEG:
                quality = 70; // JPEG quality is in [0,100] range
                comprParamID = cv::IMWRITE_JPEG_QUALITY;
                compression = ".jpg";
                break;
            case COMPRESSIONS::WEBP:
                quality = 70; // from 1 to 100 (the higher is the better). By default (without any parameter) and for quality above 100 the lossless compression is used.
                comprParamID = cv::IMWRITE_WEBP_QUALITY;
                compression = ".webp";
                break;
        }

        auto params = std::vector<int>();
        params.push_back( comprParamID );
        params.push_back( quality ); // Quality of compression

        std::vector<uchar> buf;
        cv::imencode( compression, image, buf, params );
        // we need to copy the data here as we can not tell opencv to free the buffer on demand
		cv::Mat result(buf, true);

		/*auto start = std::chrono::system_clock::now();
		auto end = std::chrono::system_clock::now();
		auto diff = end - start;
		std::cout << "duration: " << std::chrono::duration_cast<std::chrono::microseconds>(diff).count() << std::endl;*/

        return result;
    });
    D::SetNodeName(*trans.ptr_, "ColorCompression::compress");
    return trans;
}

EventsT<cv::Mat> ColorCompression::deCompress(const EventsT<cv::Mat> &inStream) {
    auto trans = Transform(inStream, [] (const cv::Mat &image) {
		cv::Mat result = cv::imdecode(image, cv::ImreadModes::IMREAD_UNCHANGED);
		return result;
    });
    D::SetNodeName(*trans.ptr_, "ColorCompression::deCompress");
    return trans;
}

std::string ColorCompression::ToString() {
    return "ColorCompression";
}


} // namespace dyncam