//
// Created by ascadian on 11/30/17.
//

#pragma once
#include "../ReactDomain.h"
#include "BaseCompression.h"
#include <opencv2/opencv.hpp>

namespace dyncam {

    enum class COMPRESSIONS {
        PNG,
        JPEG,
        WEBP
    };

class ColorCompression : public BaseCompression, std::enable_shared_from_this<ColorCompression> {

public:
    ColorCompression() = default;
    ColorCompression(COMPRESSIONS comp);

    COMPRESSIONS m_compression = COMPRESSIONS::JPEG;

    template<class Archive>
    void save(Archive& ar) const {
        ar (m_compression);
    }

    template<class Archive>
    void load(Archive& ar) {
        ar (m_compression);
    }

    EventsT<cv::Mat> compress(const EventsT<cv::Mat> &inStream) override;
    EventsT<cv::Mat> deCompress(const EventsT<cv::Mat> &inStream) override;

    std::string ToString() override;
};

} // namespace dyncam

CEREAL_REGISTER_TYPE(dyncam::ColorCompression)
CEREAL_REGISTER_POLYMORPHIC_RELATION(dyncam::BaseCompression, dyncam::ColorCompression)
CEREAL_SPECIALIZE_FOR_ALL_ARCHIVES(dyncam::ColorCompression, cereal::specialization::member_load_save)