//
// Created by ascadian on 8/9/17.
//

#pragma once

#include <opencv2/core/mat.hpp>
#include "src/Core/Serialization.h"

// http://docs.opencv.org/3.1.0/d9/d0c/group__calib3d.html#ga687a1ab946686f0d85ae0363b5af1d7b
class CamIntrinsics {

protected:
    cv::Mat intrinsics_;
    cv::Mat distCoeffs_;
    cv::Mat optimalCameraMatrix_; // from getOptimalNewCameraMatrix
	cv::Mat extrinsicParams_;


    void ReadFromile(const std::string filename);
public:
    CamIntrinsics();

    explicit CamIntrinsics(const std::string filename);

    CamIntrinsics(float fx, float fy,
                  float cx, float cy,
                  float k1, float k2, float k3,
                  float p1, float p2);


	CamIntrinsics(float fx, float fy,
				  float cx, float cy,
				  float shift_d, float shift_m, 
				  float mx_x3y0, float mx_x0y3, float mx_x2y1, float mx_x1y2,float mx_x2y0,float mx_x0y2,float mx_x1y1,float mx_x1y0,float mx_x0y1,float mx_x0y0,
				  float my_x3y0, float my_x0y3, float my_x2y1, float my_x1y2,float my_x2y0,float my_x0y2,float my_x1y1,float my_x1y0,float my_x0y1,float my_x0y0);

	CamIntrinsics(cv::Mat intrinsics,
		cv::Mat distCoeffs,
		cv::Mat optimalCameraMatrix);

	CamIntrinsics(cv::Mat intrinsics,
		cv::Mat distCoeffs,
		cv::Mat optimalCameraMatrix,
		cv::Mat extrinsicParams);

    /// intrinsics
    /// fx  0 cx
    ///  0 fy cy
    ///  0  0  1
    ///< Focal length x (pixel)
    const float get_fx() const;
    ///< Focal length y (pixel)
    const float get_fy() const;
    ///< Principal point x (pixel)
    const float get_cx() const;
    ///< Principal point y (pixel)
    const float get_cy() const;

    /// distCoeffs: (k_1, k_2, p_1, p_2[, k_3[, k_4, k_5, k_6]])
    ///< Radial distortion coefficient, 1st-order
    const float get_k1() const;
    ///< Radial distortion coefficient, 2nd-order
    const float get_k2() const;
    ///< Radial distortion coefficient, 3rd-order
    const float get_k3() const;
    ///< Tangential distortion coefficient
    const float get_p1() const;
    ///< Tangential distortion coefficient0
    const float get_p2() const;

	//extrinsics
	const float get_shift_d() const;
	const float get_shift_m() const;
	const cv::Mat get_extrinsicMatrices() const;

    const bool operator==(const CamIntrinsics &rhs) const;

    const bool operator!=(const CamIntrinsics &rhs) const;

    const void WriteToFile(const std::string filename) const;

    template <class Archive>
    void serialize(Archive & ar)
    {
        ar(intrinsics_, distCoeffs_, optimalCameraMatrix_, extrinsicParams_);
    }

};


