//
// Created by ascadian on 5/17/18.
//

#include "Stream.h"

namespace dyncam {

Stream::Stream() : stream_type_(StreamTypes::UNKNOWN)
    , id_(GenerateUniqueId()) {
}

Stream::Stream(StreamTypes type) : Stream() {
    stream_type_ = type;
}

int Stream::GetDyncamVersion() {
    return dyncamVersion_;
}

int Stream::GetId() {
    return id_;
}

int Stream::GetParentId() {
    return parent_id_;
}

int Stream::GenerateUniqueId() {
    static std::atomic<std::uint32_t> uid { 0 };
    return ++uid;
}

std::string Stream::StreamTypesToString(const StreamTypes type) {
    switch (type) {
        case StreamTypes::COLOR:             return "Color";
        case StreamTypes::IR:                return "IR";
        case StreamTypes::DEPTH:             return "Depth";
        case StreamTypes::UNDISTORTED_DEPTH: return "Undistorted Depth";
        case StreamTypes::REGISTERED_COLOR:  return "RegisteredColor";
        case StreamTypes::UNKNOWN:           return "Unknown";
        case StreamTypes::EXTRINSICS:        return "Extrinsics";
        case StreamTypes::INTRINSICS:        return "Intrinsics";
        case StreamTypes::COMPRESSED:        return "Compressed";
    }
    return "Unknown capability";
}
StreamTypes Stream::StreamTypesFromString(std::string const & value) {
    if (value == "UNKNOWN") return StreamTypes::UNKNOWN;
    else if (value == "COLOR") return StreamTypes::COLOR;
    else if (value == "IR") return StreamTypes::IR;
    else if (value == "DEPTH") return StreamTypes::DEPTH;
    else if (value == "UNDISTORTED_DEPTH") return StreamTypes::UNDISTORTED_DEPTH;
    else if (value == "REGISTERED_COLOR") return StreamTypes::REGISTERED_COLOR;
    else if (value == "EXTRINSICS") return StreamTypes::EXTRINSICS;
    else if (value == "INTRINSICS") return StreamTypes::INTRINSICS;
    else if (value == "COMPRESSED") return StreamTypes::COMPRESSED;
    else assert(false); // "Unknown Stream Type";
}
StreamTypes Stream::getType() {
    return stream_type_;
}

} // namespace dyncam