#include <QtNetwork/QHostInfo>
#include <QTimer>
#include <src/UI/Widgets/camerawidget.h>
#include "src/UI/Widgets/streamwidget.h"
#include "src/Core/ImageStream.h"
#include "src/Cams/NetCam/CompressedStream.h"
#include "src/Core/IntrinsicsStream.h"
#include "clientwindow.h"
#include "src/Compression/Lz4Compression.h"
#include "src/Compression/Lz4QuantCompression.h"
#include "src/Compression/QuantizedCompression.h"
#include "src/Compression/ColorCompression.h"
#include "ui_clientwindow.h"

ClientWindow::ClientWindow(QWidget *parent) :
        QMainWindow(parent),
        ui(new Ui::ClientWindow) {
    ui->setupUi(this);

    connect(ui->connectButton, &QPushButton::clicked,
            this, &ClientWindow::on_connect_clicked);

    connect(ui->requestButton, &QPushButton::clicked,
            this, &ClientWindow::on_request_streams_clicked);
}

ClientWindow::~ClientWindow() {
#ifdef REACT_ENABLE_LOGGING
    std::ofstream logfile;
    logfile.open("log.txt");

    D::Log().Write(logfile);
    logfile.close();
#endif

    delete ui;
}

void ClientWindow::on_connect_clicked() {
    auto client = std::make_unique<StreamClient>();
    client->start(ui->addressText->text().toStdString(),
                  ui->portText->text().toStdString());
    m_netClients.push_back(std::move(client));
}

void ClientWindow::on_request_streams_clicked() {
    auto &streams = m_netClients.front()->m_streams;

	std::shared_ptr<CompressedStream> cs_streamRC = std::make_shared<CompressedStream>(streams->GetStreamFirst<ImageStream>(StreamTypes::REGISTERED_COLOR),
		std::make_unique<ColorCompression>());

	std::shared_ptr<CompressedStream> cs_streamC = std::make_shared<CompressedStream>(streams->GetStreamFirst<ImageStream>(StreamTypes::COLOR),
	std::make_unique<ColorCompression>());

	std::shared_ptr<CompressedStream> cs_streamUD = std::make_shared<CompressedStream>(streams->GetStreamFirst<ImageStream>(StreamTypes::UNDISTORTED_DEPTH),
		std::make_unique<Lz4QuantCompression>(cv::Size(512, 424), CV_16UC1));

	std::shared_ptr<CompressedStream> cs_streamD = std::make_shared<CompressedStream>(streams->GetStreamFirst<ImageStream>(StreamTypes::DEPTH),
		std::make_unique<QuantizedCompression>());

	m_netClients.front()->requestStream(cs_streamRC);
	m_netClients.front()->requestStream(cs_streamC);
	m_netClients.front()->requestStream(cs_streamUD);
	m_netClients.front()->requestStream(cs_streamD);

	//m_netClients.front()->requestStream(streams->GetStreamFirst<ImageStream>(StreamTypes::COLOR));
    //m_netClients.front()->requestStream(streams->GetStreamFirst<ImageStream>(StreamTypes::DEPTH));
	m_netClients.front()->requestStream(streams->GetStreamFirst<ImageStream>(StreamTypes::IR));
    m_netClients.front()->requestStream(streams->GetStreamFirst<IntrinsicsStream>(StreamTypes::INTRINSICS));

    for (auto &client : m_netClients) {
        CameraWidget *camWidget = new CameraWidget(this);
        camWidget->setAllowedAreas(Qt::AllDockWidgetAreas);
        addDockWidget(Qt::RightDockWidgetArea, camWidget);
        resizeDocks({camWidget}, {300}, Qt::Horizontal);
        camWidget->setCam(*client->m_streams);
    }
}