//
// Created by ascadian on 2/21/18.
//

#include "camerawidget.h"

#include <utility>
#include "src/Core/ImageStream.h"
#include "streamwidget.h"
#include "ui_camerawidget.h"

Q_DECLARE_METATYPE(std::shared_ptr<dyncam::Stream>)

CameraWidget::CameraWidget(QWidget *parent) :
        QDockWidget(parent),
        ui(new Ui::CameraWidget) {
    ui->setupUi(this);

    // we need that ugly cast because QT... (two signals with the same name)
    connect(ui->colorCombo_2, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
            this, &CameraWidget::on_source_selected);
}

CameraWidget::~CameraWidget() {
    delete ui;
}

void CameraWidget::setCam(StreamCollection streams) {
    mCam = streams;
    setWindowTitle(QString::fromStdString(mCam.GetSerial()));

    auto depth_undist = streams.GetStreamFirst<ImageStream>(StreamTypes::UNDISTORTED_DEPTH);
    auto depth = streams.GetStreamFirst<ImageStream>(StreamTypes::DEPTH);
    if (depth || depth_undist) {
        m_pointsMapper = std::make_shared<CloudFromDepth>(mCam);
    }

    for (auto s : mCam.GetAllStreams()) {
        ui->colorCombo_2->addItem(QString::fromStdString(Stream::StreamTypesToString(s->getType())),
                                  QVariant::fromValue(s));
    }
}



void CameraWidget::on_source_selected(int index) {
    auto stream = ui->colorCombo_2->itemData(index).value<std::shared_ptr<dyncam::Stream>>();
    auto cap = stream->getType();

    switch (cap) {
        case StreamTypes::IR:
        case StreamTypes::DEPTH:
        case StreamTypes::REGISTERED_COLOR:
        case StreamTypes::COLOR:
            ui->streamWidget->setEvent(std::dynamic_pointer_cast<ImageStream>(stream)->events, cap);
            break;
        case StreamTypes::UNDISTORTED_DEPTH:
            {
                // test if we have 3d capabilities TODO: maybe we should test for intrinsics?
                auto regColor = mCam.GetStreamFirst<ImageStream>(StreamTypes::REGISTERED_COLOR);
                if (regColor) {
                    ui->streamWidget->setEventCloud(m_pointsMapper->PointPosEvents,
                                                    m_pointsMapper->PointColorsEvents);
                } else {
                    ui->streamWidget->setEventCloud(m_pointsMapper->PointPosEvents);
                }
            }
            break;

        case StreamTypes::UNKNOWN:break;
        case StreamTypes::EXTRINSICS:break;
        case StreamTypes::INTRINSICS:break;
        case StreamTypes::COMPRESSED:break;
    }
}